@@ -7,16 +7,9 @@ module Sequel
       AUTO_VACUUM = [:none, :full, :incremental].freeze
       PRIMARY_KEY_INDEX_RE = /\Asqlite_autoindex_/.freeze
       SYNCHRONOUS = [:off, :normal, :full].freeze
-      TABLES_FILTER = "type = 'table' AND NOT name = 'sqlite_sequence'"
+      TABLES_FILTER = "type = 'table' AND NOT name = 'sqlite_sequence'".freeze
       TEMP_STORE = [:default, :file, :memory].freeze
-      
-      # Run all alter_table commands in a transaction.  This is technically only
-      # needed for drop column.
-      def alter_table(name, generator=nil, &block)
-        remove_cached_schema(name)
-        generator ||= Schema::AlterTableGenerator.new(self, &block)
-        transaction{generator.operations.each{|op| alter_table_sql_list(name, [op]).flatten.each{|sql| execute_ddl(sql)}}}
-      end
+      VIEWS_FILTER = "type = 'view'".freeze
 
       # A symbol signifying the value of the auto_vacuum PRAGMA.
       def auto_vacuum
@@ -30,7 +23,20 @@ module Sequel
         value = AUTO_VACUUM.index(value) || (raise Error, "Invalid value for auto_vacuum option. Please specify one of :none, :full, :incremental.")
         pragma_set(:auto_vacuum, value)
       end
+
+      # Boolean signifying the value of the case_sensitive_likePRAGMA, or nil
+      # if not using SQLite 3.2.3+.
+      def case_sensitive_like
+        pragma_get(:case_sensitive_like).to_i == 1 if sqlite_version >= 30203
+      end
       
+      # Set the case_sensitive_like PRAGMA using the given boolean value, if using
+      # SQLite 3.2.3+.  If not using 3.2.3+, no error is raised. See pragma_set.
+      # Consider using the :case_sensitive_like Database option instead.
+      def case_sensitive_like=(value)
+        pragma_set(:case_sensitive_like, !!value ? 'on' : 'off') if sqlite_version >= 30203
+      end
+
       # SQLite uses the :sqlite database type.
       def database_type
         :sqlite
@@ -96,11 +102,29 @@ module Sequel
         end
       end
       
+      # SQLite supports CREATE TABLE IF NOT EXISTS syntax since 3.3.0.
+      def supports_create_table_if_not_exists?
+        sqlite_version >= 30300
+      end
+      
       # SQLite 3.6.8+ supports savepoints. 
       def supports_savepoints?
         sqlite_version >= 30608
       end
 
+      # Override the default setting for whether to use timezones in timestamps.
+      # For backwards compatibility, it is set to +true+ by default.
+      # Anyone wanting to use SQLite's datetime functions should set it to +false+
+      # using this method.  It's possible that the default will change in a future version,
+      # so anyone relying on timezones in timestamps should set this to +true+.
+      attr_writer :use_timestamp_timezones
+
+      # SQLite supports timezones in timestamps, since it just stores them as strings,
+      # but it breaks the usage of SQLite's datetime functions.
+      def use_timestamp_timezones?
+        defined?(@use_timestamp_timezones) ? @use_timestamp_timezones : (@use_timestamp_timezones = true)
+      end
+
       # A symbol signifying the value of the synchronous PRAGMA.
       def synchronous
         SYNCHRONOUS[pragma_get(:synchronous).to_i]
@@ -118,8 +142,7 @@ module Sequel
       # Options:
       # * :server - Set the server to use.
       def tables(opts={})
-        m = output_identifier_meth
-        metadata_dataset.from(:sqlite_master).server(opts[:server]).filter(TABLES_FILTER).map{|r| m.call(r[:name])}
+        tables_and_views(TABLES_FILTER, opts)
       end
       
       # A symbol signifying the value of the temp_store PRAGMA.
@@ -134,8 +157,32 @@ module Sequel
         pragma_set(:temp_store, value)
       end
       
+      # Array of symbols specifying the view names in the current database.
+      #
+      # Options:
+      # * :server - Set the server to use.
+      def views(opts={})
+        tables_and_views(VIEWS_FILTER, opts)
+      end
+
       private
 
+      # Run all alter_table commands in a transaction.  This is technically only
+      # needed for drop column.
+      def apply_alter_table(table, ops)
+        transaction do 
+          if ops.length > 1 && ops.all?{|op| op[:op] == :add_constraint}
+            # If you are just doing constraints, apply all of them at the same time,
+            # as otherwise all but the last one get lost.
+            alter_table_sql_list(table, [{:op=>:add_constraints, :ops=>ops}]).flatten.each{|sql| execute_ddl(sql)}
+          else
+            # Run each operation separately, as later operations may depend on the
+            # results of earlier operations.
+            ops.each{|op| alter_table_sql_list(table, [op]).flatten.each{|sql| execute_ddl(sql)}}
+          end
+        end
+      end
+
       # SQLite supports limited table modification.  You can add a column
       # or an index.  Dropping columns is supported by copying the table into
       # a temporary table, dropping the table, and creating a new table without
@@ -162,11 +209,26 @@ module Sequel
           duplicate_table(table){|columns| columns.each{|s| s[:null] = op[:null] if s[:name].to_s == op[:name].to_s}}
         when :set_column_type
           duplicate_table(table){|columns| columns.each{|s| s[:type] = op[:type] if s[:name].to_s == op[:name].to_s}}
+        when :drop_constraint
+          case op[:type]
+          when :primary_key
+            duplicate_table(table){|columns| columns.each{|s| s[:primary_key] = nil}}
+          when :foreign_key
+            duplicate_table(table){|columns| columns.each{|s| s[:table] = nil}}
+          when :unique
+            duplicate_table(table)
+          else
+            raise Error, "Unsupported :type option for drop_constraint: #{op[:type].inspect}"
+          end
+        when :add_constraint
+          duplicate_table(table, :constraints=>[op])
+        when :add_constraints
+          duplicate_table(table, :constraints=>op[:ops])
         else
-          raise Error, "Unsupported ALTER TABLE operation"
+          raise Error, "Unsupported ALTER TABLE operation: #{op[:op].inspect}"
         end
       end
-      
+
       # The array of column symbols in the table, except for ones given in opts[:except]
       def backup_table_name(table, opts={})
         table = table.gsub('`', '')
@@ -176,17 +238,14 @@ module Sequel
         end
       end
 
-      # Allow use without a generator, needed for the alter table hackery that Sequel allows.
-      def column_list_sql(generator)
-        generator.is_a?(Schema::Generator) ? super : generator.map{|c| column_definition_sql(c)}.join(', ')
-      end
-
       # Array of PRAGMA SQL statements based on the Database options that should be applied to
       # new connections.
       def connection_pragmas
         ps = []
         v = typecast_value_boolean(opts.fetch(:foreign_keys, 1))
         ps << "PRAGMA foreign_keys = #{v ? 1 : 0}"
+        v = typecast_value_boolean(opts.fetch(:case_sensitive_like, 1))
+        ps << "PRAGMA case_sensitive_like = #{v ? 1 : 0}"
         [[:auto_vacuum, AUTO_VACUUM], [:synchronous, SYNCHRONOUS], [:temp_store, TEMP_STORE]].each do |prag, con|
           if v = opts[prag]
             raise(Error, "Value for PRAGMA #{prag} not supported, should be one of #{con.join(', ')}") unless v = con.index(v.to_sym)
@@ -208,15 +267,20 @@ module Sequel
           cols.reject!{|c| nono.include? c[:name] }
         end
 
-        if foreign_keys
-          metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
-            c = cols.find {|co| co[:name] == row[:from] } or next
-            c[:table] = row[:table]
-            c[:key] = row[:to]
-            c[:on_update] = on_delete_sql_to_sym(row[:on_update])
-            c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
-            # is there any way to get deferrable status?
+        begin
+          if foreign_keys
+            metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
+              c = cols.find {|co| co[:name] == row[:from] } or next
+              c[:table] = row[:table]
+              c[:key] = row[:to]
+              c[:on_update] = on_delete_sql_to_sym(row[:on_update])
+              c[:on_delete] = on_delete_sql_to_sym(row[:on_delete])
+              # is there any way to get deferrable status?
+            end
           end
+        rescue Sequel::DatabaseError
+          # Doesn't work correctly on some versions of JDBC SQLite,
+          # giving a "query does not return ResultSet" error.
         end
         cols
       end
@@ -231,17 +295,26 @@ module Sequel
         opts[:old_columns_proc].call(old_columns) if opts[:old_columns_proc]
 
         yield def_columns if block_given?
-        def_columns_str = column_list_sql(def_columns)
+
+        constraints = (opts[:constraints] || []).dup
+        pks = []
+        def_columns.each{|c| pks << c[:name] if c[:primary_key]}
+        if pks.length > 1
+          constraints << {:type=>:primary_key, :columns=>pks}
+          def_columns.each{|c| c[:primary_key] = false if c[:primary_key]}
+        end
+
+        def_columns_str = (def_columns.map{|c| column_definition_sql(c)} + constraints.map{|c| constraint_definition_sql(c)}).join(', ')
         new_columns = old_columns.dup
         opts[:new_columns_proc].call(new_columns) if opts[:new_columns_proc]
 
         qt = quote_schema_table(table)
         bt = quote_identifier(backup_table_name(qt))
         a = [
-           "CREATE TABLE #{bt}(#{def_columns_str})",
-           "INSERT INTO #{bt}(#{dataset.send(:identifier_list, new_columns)}) SELECT #{dataset.send(:identifier_list, old_columns)} FROM #{qt}",
-           "DROP TABLE #{qt}",
-           "ALTER TABLE #{bt} RENAME TO #{qt}"
+           "ALTER TABLE #{qt} RENAME TO #{bt}",
+           "CREATE TABLE #{qt}(#{def_columns_str})",
+           "INSERT INTO #{qt}(#{dataset.send(:identifier_list, new_columns)}) SELECT #{dataset.send(:identifier_list, old_columns)} FROM #{bt}",
+           "DROP TABLE #{bt}"
         ]
         indexes(table).each do |name, h|
           if (h[:columns].map{|x| x.to_s} - new_columns).empty?
@@ -280,7 +353,7 @@ module Sequel
 
       # Parse the output of the table_info pragma
       def parse_pragma(table_name, opts)
-        metadata_dataset.with_sql("PRAGMA table_info(?)", input_identifier_meth.call(table_name)).map do |row|
+        metadata_dataset.with_sql("PRAGMA table_info(?)", input_identifier_meth(opts[:dataset]).call(table_name)).map do |row|
           row.delete(:cid)
           row[:allow_null] = row.delete(:notnull).to_i == 0
           row[:default] = row.delete(:dflt_value)
@@ -300,12 +373,18 @@ module Sequel
       # SQLite supports schema parsing using the table_info PRAGMA, so
       # parse the output of that into the format Sequel expects.
       def schema_parse_table(table_name, opts)
-        m = output_identifier_meth
+        m = output_identifier_meth(opts[:dataset])
         parse_pragma(table_name, opts).map do |row|
           [m.call(row.delete(:name)), row]
         end
       end
       
+      # Backbone of the tables and views support.
+      def tables_and_views(filter, opts)
+        m = output_identifier_meth
+        metadata_dataset.from(:sqlite_master).server(opts[:server]).filter(filter).map{|r| m.call(r[:name])}
+      end
+      
       # SQLite uses the integer data type even for bignums.  This is because they
       # are both stored internally as text, and converted when returned from
       # the database.  Using an integer type instead of bigint makes it more likely
@@ -320,7 +399,8 @@ module Sequel
       SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'distinct columns from join where group having compounds order limit')
       COMMA_SEPARATOR = ', '.freeze
       CONSTANT_MAP = {:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}
-    
+      EXTRACT_MAP = {:year=>"'%Y'", :month=>"'%m'", :day=>"'%d'", :hour=>"'%H'", :minute=>"'%M'", :second=>"'%f'"}
+
       # SQLite does not support pattern matching via regular expressions.
       # SQLite is case insensitive (depending on pragma), so use LIKE for
       # ILIKE.
@@ -328,9 +408,21 @@ module Sequel
         case op
         when :~, :'!~', :'~*', :'!~*'
           raise Error, "SQLite does not support pattern matching via regular expressions"
-        when :LIKE, :'NOT LIKE', :ILIKE, :'NOT ILIKE'
-          # SQLite is case insensitive for ASCII, and non case sensitive for other character sets
-          "#{'NOT ' if [:'NOT LIKE', :'NOT ILIKE'].include?(op)}(#{literal(args.at(0))} LIKE #{literal(args.at(1))})"
+        when :ILIKE
+          super(:LIKE, args.map{|a| SQL::Function.new(:upper, a)})
+        when :"NOT LIKE", :"NOT ILIKE"
+          "NOT #{complex_expression_sql((op == :"NOT ILIKE" ? :ILIKE : :LIKE), args)}"
+        when :^
+          complex_expression_arg_pairs(args) do |a, b|
+            a = literal(a)
+            b = literal(b)
+            "((~(#{a} & #{b})) & (#{a} | #{b}))"
+          end
+        when :extract
+          part = args.at(0)
+          raise(Sequel::Error, "unsupported extract argument: #{part.inspect}") unless format = EXTRACT_MAP[part]
+          expr = args.at(1)
+          "CAST(strftime(#{format}, #{literal(expr)}) AS #{part == :second ? 'NUMERIC' : 'INTEGER'})"
         else
           super(op, args)
         end
@@ -366,6 +458,18 @@ module Sequel
         "`#{c}`"
       end
       
+      # When a qualified column is selected on SQLite and the qualifier
+      # is a subselect, the column name used is the full qualified name
+      # (including the qualifier) instead of just the column name.  To
+      # get correct column names, you must use an alias.
+      def select(*cols)
+        if ((f = @opts[:from]) && f.any?{|t| t.is_a?(Dataset) || (t.is_a?(SQL::AliasedExpression) && t.expression.is_a?(Dataset))}) || ((j = @opts[:join]) && j.any?{|t| t.table.is_a?(Dataset)})
+          super(*cols.map{|c| alias_qualified_column(c)})
+        else
+          super
+        end
+      end
+      
       # SQLite does not support INTERSECT ALL or EXCEPT ALL
       def supports_intersect_except_all?
         false
@@ -382,11 +486,17 @@ module Sequel
       end
       
       # SQLite supports timezones in literal timestamps, since it stores them
-      # as text.
+      # as text.  But using timezones in timestamps breaks SQLite datetime
+      # functions, so we allow the user to override the default per database.
       def supports_timestamp_timezones?
-        true
+        db.use_timestamp_timezones?
       end
 
+      # SQLite cannot use WHERE 't'.
+      def supports_where_true?
+        false
+      end
+      
       private
       
       # SQLite uses string literals instead of identifiers in AS clauses.
@@ -394,6 +504,23 @@ module Sequel
         aliaz = aliaz.value if aliaz.is_a?(SQL::Identifier)
         "#{expression} AS #{literal(aliaz.to_s)}"
       end
+
+      # If col is a qualified column, alias it to the same as the column name
+      def alias_qualified_column(col)
+        case col
+        when Symbol
+          t, c, a = split_symbol(col)
+          if t && !a
+            alias_qualified_column(SQL::QualifiedIdentifier.new(t, c))
+          else
+            col
+          end
+        when SQL::QualifiedIdentifier
+          SQL::AliasedExpression.new(col, col.column)
+        else
+          col
+        end
+      end
       
       # SQL fragment specifying a list of identifiers
       def identifier_list(columns)
@@ -406,7 +533,7 @@ module Sequel
         v.each_byte{|x| blob << sprintf('%02x', x)}
         "X'#{blob}'"
       end
-      
+
       # SQLite does not support the SQL WITH clause
       def select_clause_methods
         SELECT_CLAUSE_METHODS
@@ -416,7 +543,7 @@ module Sequel
       def select_lock_sql(sql)
         super unless @opts[:lock] == :update
       end
-      
+
       # SQLite treats a DELETE with no WHERE clause as a TRUNCATE
       def _truncate_sql(table)
         "DELETE FROM #{table}"
