@@ -34,7 +34,7 @@ SQL_BEGIN = 'BEGIN'
 SQL_ROLLBACK = 'ROLLBACK'
 SQL_COMMIT = 'COMMIT'
 
-context "MySQL", '#create_table' do
+describe "MySQL", '#create_table' do
   before do
     @db = MYSQL_DB
     MYSQL_DB.sqls.clear
@@ -45,22 +45,22 @@ context "MySQL", '#create_table' do
   
   specify "should allow to specify options for MySQL" do
     @db.create_table(:dolls, :engine => 'MyISAM', :charset => 'latin2'){text :name}
-    @db.sqls.should == ["CREATE TABLE dolls (name text) ENGINE=MyISAM DEFAULT CHARSET=latin2"]
+    @db.sqls.should == ["CREATE TABLE `dolls` (`name` text) ENGINE=MyISAM DEFAULT CHARSET=latin2"]
   end
   
   specify "should create a temporary table" do
     @db.create_table(:tmp_dolls, :temp => true, :engine => 'MyISAM', :charset => 'latin2'){text :name}
-    @db.sqls.should == ["CREATE TEMPORARY TABLE tmp_dolls (name text) ENGINE=MyISAM DEFAULT CHARSET=latin2"]
+    @db.sqls.should == ["CREATE TEMPORARY TABLE `tmp_dolls` (`name` text) ENGINE=MyISAM DEFAULT CHARSET=latin2"]
   end
   
   specify "should not use a default for a String :text=>true type" do
     @db.create_table(:dolls){String :name, :text=>true, :default=>'blah'}
-    @db.sqls.should == ["CREATE TABLE dolls (name text)"]
+    @db.sqls.should == ["CREATE TABLE `dolls` (`name` text)"]
   end
   
   specify "should not use a default for a File type" do
     @db.create_table(:dolls){File :name, :default=>'blah'}
-    @db.sqls.should == ["CREATE TABLE dolls (name blob)"]
+    @db.sqls.should == ["CREATE TABLE `dolls` (`name` blob)"]
   end
   
   specify "should respect the size option for File type" do
@@ -82,9 +82,15 @@ context "MySQL", '#create_table' do
     end
     @db.schema(:dolls).map{|k, v| v[:auto_increment]}.should == [nil, nil, true]
   end
+
+  specify "should support collate with various other column options" do
+    @db.create_table!(:dolls){ String :name, :size=>128, :collate=>:utf8_bin, :default=>'foo', :null=>false, :unique=>true}
+    @db[:dolls].insert
+    @db[:dolls].select_map(:name).should == ["foo"]
+  end
 end
 
-context "A MySQL database" do
+describe "A MySQL database" do
   specify "should provide the server version" do
     MYSQL_DB.server_version.should >= 40000
   end
@@ -99,25 +105,25 @@ context "A MySQL database" do
 end
 
 if MYSQL_DB.adapter_scheme == :mysql
-  context "Sequel::MySQL.convert_tinyint_to_bool" do
+  describe "Sequel::MySQL.convert_tinyint_to_bool" do
     before do
       @db = MYSQL_DB
       @db.create_table(:booltest){column :b, 'tinyint(1)'; column :i, 'tinyint(4)'}
       @ds = @db[:booltest]
     end
     after do
-      Sequel::MySQL.convert_tinyint_to_bool = true
+      @db.convert_tinyint_to_bool = true
       @db.drop_table(:booltest)
     end
     
     specify "should consider tinyint(1) datatypes as boolean if set, but not larger tinyints" do
       @db.schema(:booltest, :reload=>true).should == [[:b, {:type=>:boolean, :allow_null=>true, :primary_key=>false, :default=>nil, :ruby_default=>nil, :db_type=>"tinyint(1)"}, ], [:i, {:type=>:integer, :allow_null=>true, :primary_key=>false, :default=>nil, :ruby_default=>nil, :db_type=>"tinyint(4)"}, ]]
-      Sequel::MySQL.convert_tinyint_to_bool = false
+      @db.convert_tinyint_to_bool = false
       @db.schema(:booltest, :reload=>true).should == [[:b, {:type=>:integer, :allow_null=>true, :primary_key=>false, :default=>nil, :ruby_default=>nil, :db_type=>"tinyint(1)"}, ], [:i, {:type=>:integer, :allow_null=>true, :primary_key=>false, :default=>nil, :ruby_default=>nil, :db_type=>"tinyint(4)"}, ]]
     end
     
     specify "should return tinyint(1)s as bools and tinyint(4)s as integers when set" do
-      Sequel::MySQL.convert_tinyint_to_bool = true
+      @db.convert_tinyint_to_bool = true
       @ds.delete
       @ds << {:b=>true, :i=>10}
       @ds.all.should == [{:b=>true, :i=>10}]
@@ -130,7 +136,7 @@ if MYSQL_DB.adapter_scheme == :mysql
     end
 
     specify "should return all tinyints as integers when unset" do
-      Sequel::MySQL.convert_tinyint_to_bool = false
+      @db.convert_tinyint_to_bool = false
       @ds.delete
       @ds << {:b=>true, :i=>10}
       @ds.all.should == [{:b=>1, :i=>10}]
@@ -148,7 +154,7 @@ if MYSQL_DB.adapter_scheme == :mysql
   end
 end
 
-context "A MySQL dataset" do
+describe "A MySQL dataset" do
   before do
     MYSQL_DB.create_table(:items){String :name; Integer :value}
     @d = MYSQL_DB[:items]
@@ -220,12 +226,12 @@ context "A MySQL dataset" do
   
   specify "should support ORDER clause in UPDATE statements" do
     @d.order(:name).update_sql(:value => 1).should == \
-      'UPDATE items SET value = 1 ORDER BY name'
+      'UPDATE `items` SET `value` = 1 ORDER BY `name`'
   end
   
   specify "should support LIMIT clause in UPDATE statements" do
     @d.limit(10).update_sql(:value => 1).should == \
-      'UPDATE items SET value = 1 LIMIT 10'
+      'UPDATE `items` SET `value` = 1 LIMIT 10'
   end
   
   specify "should support regexps" do
@@ -243,7 +249,7 @@ context "A MySQL dataset" do
   end
 end
 
-context "MySQL datasets" do
+describe "MySQL datasets" do
   before do
     @d = MYSQL_DB[:orders]
   end
@@ -282,7 +288,7 @@ describe "Dataset#distinct" do
   end
 end
 
-context "MySQL join expressions" do
+describe "MySQL join expressions" do
   before do
     @ds = MYSQL_DB[:nodes]
     @ds.db.meta_def(:server_version) {50014}
@@ -293,47 +299,47 @@ context "MySQL join expressions" do
   end
   specify "should support natural left joins" do
     @ds.join_table(:natural_left, :nodes).sql.should == \
-      'SELECT * FROM nodes NATURAL LEFT JOIN nodes'
+      'SELECT * FROM `nodes` NATURAL LEFT JOIN `nodes`'
   end
   specify "should support natural right joins" do
     @ds.join_table(:natural_right, :nodes).sql.should == \
-      'SELECT * FROM nodes NATURAL RIGHT JOIN nodes'
+      'SELECT * FROM `nodes` NATURAL RIGHT JOIN `nodes`'
   end
   specify "should support natural left outer joins" do
     @ds.join_table(:natural_left_outer, :nodes).sql.should == \
-      'SELECT * FROM nodes NATURAL LEFT OUTER JOIN nodes'
+      'SELECT * FROM `nodes` NATURAL LEFT OUTER JOIN `nodes`'
   end
   specify "should support natural right outer joins" do
     @ds.join_table(:natural_right_outer, :nodes).sql.should == \
-      'SELECT * FROM nodes NATURAL RIGHT OUTER JOIN nodes'
+      'SELECT * FROM `nodes` NATURAL RIGHT OUTER JOIN `nodes`'
   end
   specify "should support natural inner joins" do
     @ds.join_table(:natural_inner, :nodes).sql.should == \
-      'SELECT * FROM nodes NATURAL LEFT JOIN nodes'
+      'SELECT * FROM `nodes` NATURAL LEFT JOIN `nodes`'
   end
   specify "should support cross joins" do
     @ds.join_table(:cross, :nodes).sql.should == \
-      'SELECT * FROM nodes CROSS JOIN nodes'
+      'SELECT * FROM `nodes` CROSS JOIN `nodes`'
   end
   specify "should support cross joins as inner joins if conditions are used" do
     @ds.join_table(:cross, :nodes, :id=>:id).sql.should == \
-      'SELECT * FROM nodes INNER JOIN nodes ON (nodes.id = nodes.id)'
+      'SELECT * FROM `nodes` INNER JOIN `nodes` ON (`nodes`.`id` = `nodes`.`id`)'
   end
   specify "should support straight joins (force left table to be read before right)" do
     @ds.join_table(:straight, :nodes).sql.should == \
-      'SELECT * FROM nodes STRAIGHT_JOIN nodes'
+      'SELECT * FROM `nodes` STRAIGHT_JOIN `nodes`'
   end
   specify "should support natural joins on multiple tables." do
     @ds.join_table(:natural_left_outer, [:nodes, :branches]).sql.should == \
-      'SELECT * FROM nodes NATURAL LEFT OUTER JOIN (nodes, branches)'
+      'SELECT * FROM `nodes` NATURAL LEFT OUTER JOIN (`nodes`, `branches`)'
   end
   specify "should support straight joins on multiple tables." do
     @ds.join_table(:straight, [:nodes,:branches]).sql.should == \
-      'SELECT * FROM nodes STRAIGHT_JOIN (nodes, branches)'
+      'SELECT * FROM `nodes` STRAIGHT_JOIN (`nodes`, `branches`)'
   end
 end
 
-context "Joined MySQL dataset" do
+describe "Joined MySQL dataset" do
   before do
     @ds = MYSQL_DB[:nodes]
   end
@@ -348,16 +354,16 @@ context "Joined MySQL dataset" do
     proc {@ds.having('blah')}.should_not raise_error
 
     @ds.having('blah').sql.should == \
-      "SELECT * FROM nodes HAVING (blah)"
+      "SELECT * FROM `nodes` HAVING (blah)"
   end
   
   specify "should put a having clause before an order by clause" do
     @ds.order(:aaa).having(:bbb => :ccc).sql.should == \
-      "SELECT * FROM nodes HAVING (bbb = ccc) ORDER BY aaa"
+      "SELECT * FROM `nodes` HAVING (`bbb` = `ccc`) ORDER BY `aaa`"
   end
 end
 
-context "A MySQL database" do
+describe "A MySQL database" do
   before do
     @db = MYSQL_DB
   end
@@ -424,7 +430,7 @@ context "A MySQL database" do
   end
 end  
 
-context "A MySQL database with table options" do
+describe "A MySQL database with table options" do
   before do
     @options = {:engine=>'MyISAM', :charset=>'latin1', :collate => 'latin1_swedish_ci'}
     
@@ -447,21 +453,21 @@ context "A MySQL database with table options" do
   
   specify "should allow to pass custom options (engine, charset, collate) for table creation" do
     @db.create_table(:items, @options){Integer :size; text :name}
-    @db.sqls.should == ["CREATE TABLE items (size integer, name text) ENGINE=MyISAM DEFAULT CHARSET=latin1 DEFAULT COLLATE=latin1_swedish_ci"]
+    @db.sqls.should == ["CREATE TABLE `items` (`size` integer, `name` text) ENGINE=MyISAM DEFAULT CHARSET=latin1 DEFAULT COLLATE=latin1_swedish_ci"]
   end
   
   specify "should use default options if specified (engine, charset, collate) for table creation" do
     @db.create_table(:items){Integer :size; text :name}
-    @db.sqls.should == ["CREATE TABLE items (size integer, name text) ENGINE=InnoDB DEFAULT CHARSET=utf8 DEFAULT COLLATE=utf8_general_ci"]
+    @db.sqls.should == ["CREATE TABLE `items` (`size` integer, `name` text) ENGINE=InnoDB DEFAULT CHARSET=utf8 DEFAULT COLLATE=utf8_general_ci"]
   end
   
   specify "should not use default if option has a nil value" do
     @db.create_table(:items, :engine=>nil, :charset=>nil, :collate=>nil){Integer :size; text :name}
-    @db.sqls.should == ["CREATE TABLE items (size integer, name text)"]
+    @db.sqls.should == ["CREATE TABLE `items` (`size` integer, `name` text)"]
   end
 end
 
-context "A MySQL database" do
+describe "A MySQL database" do
   before do
     @db = MYSQL_DB
     @db.drop_table(:items) rescue nil
@@ -473,24 +479,24 @@ context "A MySQL database" do
   
   specify "should support defaults for boolean columns" do
     @db.create_table(:items){TrueClass :active1, :default=>true; FalseClass :active2, :default => false}
-    @db.sqls.should == ["CREATE TABLE items (active1 tinyint(1) DEFAULT 1, active2 tinyint(1) DEFAULT 0)"]
+    @db.sqls.should == ["CREATE TABLE `items` (`active1` tinyint(1) DEFAULT 1, `active2` tinyint(1) DEFAULT 0)"]
   end
   
   specify "should correctly format CREATE TABLE statements with foreign keys" do
     @db.create_table(:items){Integer :id; foreign_key :p_id, :items, :key => :id, :null => false, :on_delete => :cascade}
-    @db.sqls.should == ["CREATE TABLE items (id integer, p_id integer NOT NULL, FOREIGN KEY (p_id) REFERENCES items(id) ON DELETE CASCADE)"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer, `p_id` integer NOT NULL, FOREIGN KEY (`p_id`) REFERENCES `items`(`id`) ON DELETE CASCADE)"]
   end
   
   specify "should correctly format ALTER TABLE statements with foreign keys" do
     @db.create_table(:items){Integer :id}
     @db.alter_table(:items){add_foreign_key :p_id, :users, :key => :id, :null => false, :on_delete => :cascade}
-    @db.sqls.should == ["CREATE TABLE items (id integer)", "ALTER TABLE items ADD COLUMN p_id integer NOT NULL", "ALTER TABLE items ADD FOREIGN KEY (p_id) REFERENCES users(id) ON DELETE CASCADE"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer)", "ALTER TABLE `items` ADD COLUMN `p_id` integer NOT NULL", "ALTER TABLE `items` ADD FOREIGN KEY (`p_id`) REFERENCES `users`(`id`) ON DELETE CASCADE"]
   end
   
   specify "should have rename_column support keep existing options" do
     @db.create_table(:items){String :id, :null=>false, :default=>'blah'}
     @db.alter_table(:items){rename_column :id, :nid}
-    @db.sqls.should == ["CREATE TABLE items (id varchar(255) NOT NULL DEFAULT 'blah')", "DESCRIBE items", "ALTER TABLE items CHANGE COLUMN id nid varchar(255) NOT NULL DEFAULT 'blah'"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` varchar(255) NOT NULL DEFAULT 'blah')", "DESCRIBE `items`", "ALTER TABLE `items` CHANGE COLUMN `id` `nid` varchar(255) NOT NULL DEFAULT 'blah'"]
     @db[:items].insert
     @db[:items].all.should == [{:nid=>'blah'}]
     proc{@db[:items].insert(:nid=>nil)}.should raise_error(Sequel::DatabaseError)
@@ -499,7 +505,7 @@ context "A MySQL database" do
   specify "should have set_column_type support keep existing options" do
     @db.create_table(:items){Integer :id, :null=>false, :default=>5}
     @db.alter_table(:items){set_column_type :id, Bignum}
-    @db.sqls.should == ["CREATE TABLE items (id integer NOT NULL DEFAULT 5)", "DESCRIBE items", "ALTER TABLE items CHANGE COLUMN id id bigint NOT NULL DEFAULT 5"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer NOT NULL DEFAULT 5)", "DESCRIBE `items`", "ALTER TABLE `items` CHANGE COLUMN `id` `id` bigint NOT NULL DEFAULT 5"]
     @db[:items].insert
     @db[:items].all.should == [{:id=>5}]
     proc{@db[:items].insert(:id=>nil)}.should raise_error(Sequel::DatabaseError)
@@ -511,13 +517,13 @@ context "A MySQL database" do
   specify "should have set_column_type pass through options" do
     @db.create_table(:items){integer :id; enum :list, :elements=>%w[one]}
     @db.alter_table(:items){set_column_type :id, :int, :unsigned=>true, :size=>8; set_column_type :list, :enum, :elements=>%w[two]}
-    @db.sqls.should == ["CREATE TABLE items (id integer, list enum('one'))", "DESCRIBE items", "ALTER TABLE items CHANGE COLUMN id id int(8) UNSIGNED NULL", "ALTER TABLE items CHANGE COLUMN list list enum('two') NULL"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer, `list` enum('one'))", "DESCRIBE `items`", "ALTER TABLE `items` CHANGE COLUMN `id` `id` int(8) UNSIGNED NULL", "ALTER TABLE `items` CHANGE COLUMN `list` `list` enum('two') NULL"]
   end
   
   specify "should have set_column_default support keep existing options" do
     @db.create_table(:items){Integer :id, :null=>false, :default=>5}
     @db.alter_table(:items){set_column_default :id, 6}
-    @db.sqls.should == ["CREATE TABLE items (id integer NOT NULL DEFAULT 5)", "DESCRIBE items", "ALTER TABLE items CHANGE COLUMN id id int(11) NOT NULL DEFAULT 6"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer NOT NULL DEFAULT 5)", "DESCRIBE `items`", "ALTER TABLE `items` CHANGE COLUMN `id` `id` int(11) NOT NULL DEFAULT 6"]
     @db[:items].insert
     @db[:items].all.should == [{:id=>6}]
     proc{@db[:items].insert(:id=>nil)}.should raise_error(Sequel::DatabaseError)
@@ -526,7 +532,7 @@ context "A MySQL database" do
   specify "should have set_column_allow_null support keep existing options" do
     @db.create_table(:items){Integer :id, :null=>false, :default=>5}
     @db.alter_table(:items){set_column_allow_null :id, true}
-    @db.sqls.should == ["CREATE TABLE items (id integer NOT NULL DEFAULT 5)", "DESCRIBE items", "ALTER TABLE items CHANGE COLUMN id id int(11) NULL DEFAULT 5"]
+    @db.sqls.should == ["CREATE TABLE `items` (`id` integer NOT NULL DEFAULT 5)", "DESCRIBE `items`", "ALTER TABLE `items` CHANGE COLUMN `id` `id` int(11) NULL DEFAULT 5"]
     @db[:items].insert
     @db[:items].all.should == [{:id=>5}]
     proc{@db[:items].insert(:id=>nil)}.should_not
@@ -547,7 +553,7 @@ end
 
 # Socket tests should only be run if the MySQL server is on localhost
 if %w'localhost 127.0.0.1 ::1'.include?(MYSQL_URI.host) and MYSQL_DB.adapter_scheme == :mysql
-  context "A MySQL database" do
+  describe "A MySQL database" do
     specify "should accept a socket option" do
       db = Sequel.mysql(MYSQL_DB.opts[:database], :host => 'localhost', :user => MYSQL_DB.opts[:user], :password => MYSQL_DB.opts[:password], :socket => MYSQL_SOCKET_FILE)
       proc {db.test_connection}.should_not raise_error
@@ -565,7 +571,19 @@ if %w'localhost 127.0.0.1 ::1'.include?(MYSQL_URI.host) and MYSQL_DB.adapter_sch
   end
 end
 
-context "A grouped MySQL dataset" do
+describe "A MySQL database" do
+  specify "should accept a read_timeout option when connecting" do
+    db = Sequel.connect(MYSQL_DB.opts.merge(:read_timeout=>22342))
+    proc {db.test_connection}.should_not raise_error
+  end
+
+  specify "should accept a connect_timeout option when connecting" do
+    db = Sequel.connect(MYSQL_DB.opts.merge(:connect_timeout=>22342))
+    proc {db.test_connection}.should_not raise_error
+  end
+end
+
+describe "A grouped MySQL dataset" do
   before do
     MYSQL_DB[:test2].delete
     MYSQL_DB[:test2] << {:name => '11', :value => 10}
@@ -587,7 +605,7 @@ context "A grouped MySQL dataset" do
   end
 end
 
-context "A MySQL database" do
+describe "A MySQL database" do
   before do
     @db = MYSQL_DB
     @db.drop_table(:posts) rescue nil
@@ -600,9 +618,9 @@ context "A MySQL database" do
   specify "should support fulltext indexes and full_text_search" do
     @db.create_table(:posts){text :title; text :body; full_text_index :title; full_text_index [:title, :body]}
     @db.sqls.should == [
-      "CREATE TABLE posts (title text, body text)",
-      "CREATE FULLTEXT INDEX posts_title_index ON posts (title)",
-      "CREATE FULLTEXT INDEX posts_title_body_index ON posts (title, body)"
+      "CREATE TABLE `posts` (`title` text, `body` text)",
+      "CREATE FULLTEXT INDEX `posts_title_index` ON `posts` (`title`)",
+      "CREATE FULLTEXT INDEX `posts_title_body_index` ON `posts` (`title`, `body`)"
     ]
     
     @db[:posts].insert(:title=>'ruby rails', :body=>'y')
@@ -614,32 +632,32 @@ context "A MySQL database" do
     @db[:posts].full_text_search([:title, :body], ['sequel', 'ruby']).all.should == [{:title=>'sequel', :body=>'ruby'}]
     @db[:posts].full_text_search(:title, '+ruby -rails', :boolean => true).all.should == [{:title=>'ruby scooby', :body=>'x'}]
     @db.sqls.should == [
-      "SELECT * FROM posts WHERE (MATCH (title) AGAINST ('rails'))",
-      "SELECT * FROM posts WHERE (MATCH (title, body) AGAINST ('sequel ruby'))",
-      "SELECT * FROM posts WHERE (MATCH (title) AGAINST ('+ruby -rails' IN BOOLEAN MODE))"]
+      "SELECT * FROM `posts` WHERE (MATCH (`title`) AGAINST ('rails'))",
+      "SELECT * FROM `posts` WHERE (MATCH (`title`, `body`) AGAINST ('sequel ruby'))",
+      "SELECT * FROM `posts` WHERE (MATCH (`title`) AGAINST ('+ruby -rails' IN BOOLEAN MODE))"]
   end
 
   specify "should support spatial indexes" do
     @db.create_table(:posts){point :geom, :null=>false; spatial_index [:geom]}
     @db.sqls.should == [
-      "CREATE TABLE posts (geom point NOT NULL)",
-      "CREATE SPATIAL INDEX posts_geom_index ON posts (geom)"
+      "CREATE TABLE `posts` (`geom` point NOT NULL)",
+      "CREATE SPATIAL INDEX `posts_geom_index` ON `posts` (`geom`)"
     ]
   end
 
   specify "should support indexes with index type" do
     @db.create_table(:posts){Integer :id; index :id, :type => :btree}
     @db.sqls.should == [
-      "CREATE TABLE posts (id integer)",
-      "CREATE INDEX posts_id_index USING btree ON posts (id)"
+      "CREATE TABLE `posts` (`id` integer)",
+      "CREATE INDEX `posts_id_index` USING btree ON `posts` (`id`)"
     ]
   end
 
   specify "should support unique indexes with index type" do
     @db.create_table(:posts){Integer :id; index :id, :type => :btree, :unique => true}
     @db.sqls.should == [
-      "CREATE TABLE posts (id integer)",
-      "CREATE UNIQUE INDEX posts_id_index USING btree ON posts (id)"
+      "CREATE TABLE `posts` (`id` integer)",
+      "CREATE UNIQUE INDEX `posts_id_index` USING btree ON `posts` (`id`)"
     ]
   end
 
@@ -648,9 +666,15 @@ context "A MySQL database" do
     @db << "CREATE INDEX posts_id_index ON posts (id(10))"
     @db.indexes(:posts).should == {}
   end
+
+  specify "should dump partial indexes if :partial option is set to true" do
+    @db.create_table(:posts){text :id}
+    @db << "CREATE INDEX posts_id_index ON posts (id(10))"
+    @db.indexes(:posts, :partial => true).should == {:posts_id_index => {:columns => [:id], :unique => false}}
+  end
 end
 
-context "MySQL::Dataset#insert and related methods" do
+describe "MySQL::Dataset#insert and related methods" do
   before do
     MYSQL_DB.create_table(:items){String :name; Integer :value}
     @d = MYSQL_DB[:items]
@@ -662,38 +686,20 @@ context "MySQL::Dataset#insert and related methods" do
 
   specify "#insert should insert record with default values when no arguments given" do
     @d.insert
-    
-    MYSQL_DB.sqls.should == [
-      "INSERT INTO items () VALUES ()"
-    ]
-    
-    @d.all.should == [
-      {:name => nil, :value => nil}
-    ]
+    MYSQL_DB.sqls.should == ["INSERT INTO `items` () VALUES ()"]
+    @d.all.should == [{:name => nil, :value => nil}]
   end
 
   specify "#insert  should insert record with default values when empty hash given" do
     @d.insert({})
-    
-    MYSQL_DB.sqls.should == [
-      "INSERT INTO items () VALUES ()"
-    ]
-    
-    @d.all.should == [
-      {:name => nil, :value => nil}
-    ]
+    MYSQL_DB.sqls.should == ["INSERT INTO `items` () VALUES ()"]
+    @d.all.should == [{:name => nil, :value => nil}]
   end
 
   specify "#insert should insert record with default values when empty array given" do
     @d.insert []
-    
-    MYSQL_DB.sqls.should == [
-      "INSERT INTO items () VALUES ()"
-    ]
-    
-    @d.all.should == [
-      {:name => nil, :value => nil}
-    ]
+    MYSQL_DB.sqls.should == ["INSERT INTO `items` () VALUES ()"]
+    @d.all.should == [{:name => nil, :value => nil}]
   end
 
   specify "#on_duplicate_key_update should work with regular inserts" do
@@ -704,9 +710,9 @@ context "MySQL::Dataset#insert and related methods" do
     @d.on_duplicate_key_update(:name, :value => 6).insert(:name => 'def', :value => 2)
 
     MYSQL_DB.sqls.length.should == 3
-    MYSQL_DB.sqls[0].should =~ /\AINSERT INTO items \((name|value), (name|value)\) VALUES \(('abc'|1), (1|'abc')\)\z/
-    MYSQL_DB.sqls[1].should =~ /\AINSERT INTO items \((name|value), (name|value)\) VALUES \(('abc'|1), (1|'abc')\) ON DUPLICATE KEY UPDATE name=VALUES\(name\), value=6\z/
-    MYSQL_DB.sqls[2].should =~ /\AINSERT INTO items \((name|value), (name|value)\) VALUES \(('def'|2), (2|'def')\) ON DUPLICATE KEY UPDATE name=VALUES\(name\), value=6\z/
+    MYSQL_DB.sqls[0].should =~ /\AINSERT INTO `items` \(`(name|value)`, `(name|value)`\) VALUES \(('abc'|1), (1|'abc')\)\z/
+    MYSQL_DB.sqls[1].should =~ /\AINSERT INTO `items` \(`(name|value)`, `(name|value)`\) VALUES \(('abc'|1), (1|'abc')\) ON DUPLICATE KEY UPDATE `name`=VALUES\(`name`\), `value`=6\z/
+    MYSQL_DB.sqls[2].should =~ /\AINSERT INTO `items` \(`(name|value)`, `(name|value)`\) VALUES \(('def'|2), (2|'def')\) ON DUPLICATE KEY UPDATE `name`=VALUES\(`name`\), `value`=6\z/
 
     @d.all.should == [{:name => 'abc', :value => 6}, {:name => 'def', :value => 2}]
   end
@@ -716,7 +722,7 @@ context "MySQL::Dataset#insert and related methods" do
     
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT INTO items (name) VALUES ('abc'), ('def')",
+      "INSERT INTO `items` (`name`) VALUES ('abc'), ('def')",
       SQL_COMMIT
     ]
 
@@ -731,10 +737,10 @@ context "MySQL::Dataset#insert and related methods" do
 
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT INTO items (value) VALUES (1), (2)",
+      "INSERT INTO `items` (`value`) VALUES (1), (2)",
       SQL_COMMIT,
       SQL_BEGIN,
-      "INSERT INTO items (value) VALUES (3), (4)",
+      "INSERT INTO `items` (`value`) VALUES (3), (4)",
       SQL_COMMIT
     ]
     
@@ -752,10 +758,10 @@ context "MySQL::Dataset#insert and related methods" do
 
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT INTO items (value) VALUES (1), (2)",
+      "INSERT INTO `items` (`value`) VALUES (1), (2)",
       SQL_COMMIT,
       SQL_BEGIN,
-      "INSERT INTO items (value) VALUES (3), (4)",
+      "INSERT INTO `items` (`value`) VALUES (3), (4)",
       SQL_COMMIT
     ]
     
@@ -772,7 +778,7 @@ context "MySQL::Dataset#insert and related methods" do
 
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT INTO items (name, value) VALUES ('abc', 1), ('def', 2)",
+      "INSERT INTO `items` (`name`, `value`) VALUES ('abc', 1), ('def', 2)",
       SQL_COMMIT
     ]
     
@@ -787,7 +793,7 @@ context "MySQL::Dataset#insert and related methods" do
     
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT IGNORE INTO items (name) VALUES ('abc'), ('def')",
+      "INSERT IGNORE INTO `items` (`name`) VALUES ('abc'), ('def')",
       SQL_COMMIT
     ]
 
@@ -798,7 +804,7 @@ context "MySQL::Dataset#insert and related methods" do
   
   specify "#insert_ignore should add the IGNORE keyword for single inserts" do
     @d.insert_ignore.insert(:name => 'ghi')
-    MYSQL_DB.sqls.should == ["INSERT IGNORE INTO items (name) VALUES ('ghi')"]
+    MYSQL_DB.sqls.should == ["INSERT IGNORE INTO `items` (`name`) VALUES ('ghi')"]
     @d.all.should == [{:name => 'ghi', :value => nil}]
   end
   
@@ -806,9 +812,9 @@ context "MySQL::Dataset#insert and related methods" do
     @d.on_duplicate_key_update.import([:name,:value], [['abc', 1], ['def',2]])
     
     MYSQL_DB.sqls.should == [
-      "SELECT * FROM items LIMIT 1",
+      "SELECT * FROM `items` LIMIT 1",
       SQL_BEGIN,
-      "INSERT INTO items (name, value) VALUES ('abc', 1), ('def', 2) ON DUPLICATE KEY UPDATE name=VALUES(name), value=VALUES(value)",
+      "INSERT INTO `items` (`name`, `value`) VALUES ('abc', 1), ('def', 2) ON DUPLICATE KEY UPDATE `name`=VALUES(`name`), `value`=VALUES(`value`)",
       SQL_COMMIT
     ]
 
@@ -824,7 +830,7 @@ context "MySQL::Dataset#insert and related methods" do
     
     MYSQL_DB.sqls.should == [
       SQL_BEGIN,
-      "INSERT INTO items (name, value) VALUES ('abc', 1), ('def', 2) ON DUPLICATE KEY UPDATE value=VALUES(value)",
+      "INSERT INTO `items` (`name`, `value`) VALUES ('abc', 1), ('def', 2) ON DUPLICATE KEY UPDATE `value`=VALUES(`value`)",
       SQL_COMMIT
     ]
 
@@ -835,7 +841,7 @@ context "MySQL::Dataset#insert and related methods" do
   
 end
 
-context "MySQL::Dataset#replace" do
+describe "MySQL::Dataset#replace" do
   before do
     MYSQL_DB.create_table(:items){Integer :id, :unique=>true; Integer :value}
     @d = MYSQL_DB[:items]
@@ -877,36 +883,63 @@ context "MySQL::Dataset#replace" do
   end
 end
 
-context "MySQL::Dataset#complex_expression_sql" do
+describe "MySQL::Dataset#complex_expression_sql" do
   before do
     @d = MYSQL_DB.dataset
   end
 
   specify "should handle pattern matches correctly" do
-    @d.literal(:x.like('a')).should == "(x LIKE BINARY 'a')"
-    @d.literal(~:x.like('a')).should == "(x NOT LIKE BINARY 'a')"
-    @d.literal(:x.ilike('a')).should == "(x LIKE 'a')"
-    @d.literal(~:x.ilike('a')).should == "(x NOT LIKE 'a')"
-    @d.literal(:x.like(/a/)).should == "(x REGEXP BINARY 'a')"
-    @d.literal(~:x.like(/a/)).should == "(x NOT REGEXP BINARY 'a')"
-    @d.literal(:x.like(/a/i)).should == "(x REGEXP 'a')"
-    @d.literal(~:x.like(/a/i)).should == "(x NOT REGEXP 'a')"
+    @d.literal(:x.like('a')).should == "(`x` LIKE BINARY 'a')"
+    @d.literal(~:x.like('a')).should == "(`x` NOT LIKE BINARY 'a')"
+    @d.literal(:x.ilike('a')).should == "(`x` LIKE 'a')"
+    @d.literal(~:x.ilike('a')).should == "(`x` NOT LIKE 'a')"
+    @d.literal(:x.like(/a/)).should == "(`x` REGEXP BINARY 'a')"
+    @d.literal(~:x.like(/a/)).should == "(`x` NOT REGEXP BINARY 'a')"
+    @d.literal(:x.like(/a/i)).should == "(`x` REGEXP 'a')"
+    @d.literal(~:x.like(/a/i)).should == "(`x` NOT REGEXP 'a')"
   end
 
   specify "should handle string concatenation with CONCAT if more than one record" do
-    @d.literal([:x, :y].sql_string_join).should == "CONCAT(x, y)"
-    @d.literal([:x, :y].sql_string_join(' ')).should == "CONCAT(x, ' ', y)"
-    @d.literal([:x.sql_function(:y), 1, 'z'.lit].sql_string_join(:y.sql_subscript(1))).should == "CONCAT(x(y), y[1], '1', y[1], z)"
+    @d.literal([:x, :y].sql_string_join).should == "CONCAT(`x`, `y`)"
+    @d.literal([:x, :y].sql_string_join(' ')).should == "CONCAT(`x`, ' ', `y`)"
+    @d.literal([:x.sql_function(:y), 1, 'z'.lit].sql_string_join(:y.sql_subscript(1))).should == "CONCAT(x(`y`), `y`[1], '1', `y`[1], z)"
   end
 
   specify "should handle string concatenation as simple string if just one record" do
-    @d.literal([:x].sql_string_join).should == "x"
-    @d.literal([:x].sql_string_join(' ')).should == "x"
+    @d.literal([:x].sql_string_join).should == "`x`"
+    @d.literal([:x].sql_string_join(' ')).should == "`x`"
   end
 end
 
-if MYSQL_DB.adapter_scheme == :mysql or MYSQL_DB.adapter_scheme == :jdbc
-  context "MySQL Stored Procedures" do
+describe "MySQL::Dataset#calc_found_rows" do
+  before do
+    MYSQL_DB.create_table!(:items){Integer :a}
+  end
+  after do
+    MYSQL_DB.drop_table(:items)
+  end
+
+  specify "should add the SQL_CALC_FOUND_ROWS keyword when selecting" do
+    MYSQL_DB[:items].select(:a).calc_found_rows.limit(1).sql.should == \
+      'SELECT SQL_CALC_FOUND_ROWS `a` FROM `items` LIMIT 1'
+  end
+
+  specify "should count matching rows disregarding LIMIT clause" do
+    MYSQL_DB[:items].multi_insert([{:a => 1}, {:a => 1}, {:a => 2}])
+    MYSQL_DB.sqls.clear
+
+    MYSQL_DB[:items].calc_found_rows.filter(:a => 1).limit(1).all.should == [{:a => 1}]
+    MYSQL_DB.dataset.select(:FOUND_ROWS.sql_function.as(:rows)).all.should == [{:rows => 2 }]
+
+    MYSQL_DB.sqls.should == [
+      'SELECT SQL_CALC_FOUND_ROWS * FROM `items` WHERE (`a` = 1) LIMIT 1',
+      'SELECT FOUND_ROWS() AS `rows`',
+    ]
+  end
+end
+
+if MYSQL_DB.adapter_scheme == :mysql or MYSQL_DB.adapter_scheme == :jdbc or MYSQL_DB.adapter_scheme == :mysql2
+  describe "MySQL Stored Procedures" do
     before do
       MYSQL_DB.create_table(:items){Integer :id; Integer :value}
       @d = MYSQL_DB[:items]
@@ -926,63 +959,74 @@ if MYSQL_DB.adapter_scheme == :mysql or MYSQL_DB.adapter_scheme == :jdbc
       MYSQL_DB[:items].count.should == 0
     end
     
-    specify "should be callable on the dataset object" do
-      MYSQL_DB.execute_ddl('CREATE PROCEDURE test_sproc(a INTEGER) BEGIN SELECT *, a AS b FROM items; END')
-      MYSQL_DB[:items].delete
-      @d = MYSQL_DB[:items]
-      @d.call_sproc(:select, :test_sproc, 3).should == []
-      @d.insert(:value=>1)
-      @d.call_sproc(:select, :test_sproc, 4).should == [{:id=>nil, :value=>1, :b=>4}]
-      @d.row_proc = proc{|r| r.keys.each{|k| r[k] *= 2 if r[k].is_a?(Integer)}; r}
-      @d.call_sproc(:select, :test_sproc, 3).should == [{:id=>nil, :value=>2, :b=>6}]
+    # Mysql2 doesn't support stored procedures that return result sets, probably because
+    # CLIENT_MULTI_RESULTS is not set.
+    unless MYSQL_DB.adapter_scheme == :mysql2
+      specify "should be callable on the dataset object" do
+        MYSQL_DB.execute_ddl('CREATE PROCEDURE test_sproc(a INTEGER) BEGIN SELECT *, a AS b FROM items; END')
+        MYSQL_DB[:items].delete
+        @d = MYSQL_DB[:items]
+        @d.call_sproc(:select, :test_sproc, 3).should == []
+        @d.insert(:value=>1)
+        @d.call_sproc(:select, :test_sproc, 4).should == [{:id=>nil, :value=>1, :b=>4}]
+        @d.row_proc = proc{|r| r.keys.each{|k| r[k] *= 2 if r[k].is_a?(Integer)}; r}
+        @d.call_sproc(:select, :test_sproc, 3).should == [{:id=>nil, :value=>2, :b=>6}]
+      end
+      
+      specify "should be callable on the dataset object with multiple arguments" do
+        MYSQL_DB.execute_ddl('CREATE PROCEDURE test_sproc(a INTEGER, c INTEGER) BEGIN SELECT *, a AS b, c AS d FROM items; END')
+        MYSQL_DB[:items].delete
+        @d = MYSQL_DB[:items]
+        @d.call_sproc(:select, :test_sproc, 3, 4).should == []
+        @d.insert(:value=>1)
+        @d.call_sproc(:select, :test_sproc, 4, 5).should == [{:id=>nil, :value=>1, :b=>4, :d=>5}]
+        @d.row_proc = proc{|r| r.keys.each{|k| r[k] *= 2 if r[k].is_a?(Integer)}; r}
+        @d.call_sproc(:select, :test_sproc, 3, 4).should == [{:id=>nil, :value=>2, :b=>6, :d => 8}]
+      end
     end
-    
-    specify "should be callable on the dataset object with multiple arguments" do
-      MYSQL_DB.execute_ddl('CREATE PROCEDURE test_sproc(a INTEGER, c INTEGER) BEGIN SELECT *, a AS b, c AS d FROM items; END')
+
+    specify "should deal with nil values" do
+      MYSQL_DB.execute_ddl('CREATE PROCEDURE test_sproc(i INTEGER, v INTEGER) BEGIN INSERT INTO items VALUES (i, v); END')
       MYSQL_DB[:items].delete
-      @d = MYSQL_DB[:items]
-      @d.call_sproc(:select, :test_sproc, 3, 4).should == []
-      @d.insert(:value=>1)
-      @d.call_sproc(:select, :test_sproc, 4, 5).should == [{:id=>nil, :value=>1, :b=>4, :d=>5}]
-      @d.row_proc = proc{|r| r.keys.each{|k| r[k] *= 2 if r[k].is_a?(Integer)}; r}
-      @d.call_sproc(:select, :test_sproc, 3, 4).should == [{:id=>nil, :value=>2, :b=>6, :d => 8}]
+      MYSQL_DB.call_sproc(:test_sproc, :args=>[1, nil])
+      MYSQL_DB[:items].all.should == [{:id=>1, :value=>nil}]
     end
   end
 end
 
 if MYSQL_DB.adapter_scheme == :mysql
-  context "MySQL bad date/time conversions" do
+  describe "MySQL bad date/time conversions" do
     after do
-      Sequel::MySQL.convert_invalid_date_time = false
+      MYSQL_DB.convert_invalid_date_time = false
     end
   
     specify "should raise an exception when a bad date/time is used and convert_invalid_date_time is false" do
-      Sequel::MySQL.convert_invalid_date_time = false
+      MYSQL_DB.convert_invalid_date_time = false
       proc{MYSQL_DB["SELECT CAST('0000-00-00' AS date)"].single_value}.should raise_error(Sequel::InvalidValue)
       proc{MYSQL_DB["SELECT CAST('0000-00-00 00:00:00' AS datetime)"].single_value}.should raise_error(Sequel::InvalidValue)
       proc{MYSQL_DB["SELECT CAST('25:00:00' AS time)"].single_value}.should raise_error(Sequel::InvalidValue)
     end
   
     specify "should not use a nil value bad date/time is used and convert_invalid_date_time is nil or :nil" do
-      Sequel::MySQL.convert_invalid_date_time = nil
+      MYSQL_DB.convert_invalid_date_time = nil
       MYSQL_DB["SELECT CAST('0000-00-00' AS date)"].single_value.should == nil
       MYSQL_DB["SELECT CAST('0000-00-00 00:00:00' AS datetime)"].single_value.should == nil
       MYSQL_DB["SELECT CAST('25:00:00' AS time)"].single_value.should == nil
-      Sequel::MySQL.convert_invalid_date_time = :nil
+      MYSQL_DB.convert_invalid_date_time = :nil
       MYSQL_DB["SELECT CAST('0000-00-00' AS date)"].single_value.should == nil
       MYSQL_DB["SELECT CAST('0000-00-00 00:00:00' AS datetime)"].single_value.should == nil
       MYSQL_DB["SELECT CAST('25:00:00' AS time)"].single_value.should == nil
     end
   
     specify "should not use a nil value bad date/time is used and convert_invalid_date_time is :string" do
-      Sequel::MySQL.convert_invalid_date_time = :string
+      MYSQL_DB.convert_invalid_date_time = :string
       MYSQL_DB["SELECT CAST('0000-00-00' AS date)"].single_value.should == '0000-00-00'
       MYSQL_DB["SELECT CAST('0000-00-00 00:00:00' AS datetime)"].single_value.should == '0000-00-00 00:00:00'
       MYSQL_DB["SELECT CAST('25:00:00' AS time)"].single_value.should == '25:00:00'
     end
   end
   
-  context "MySQL multiple result sets" do
+  describe "MySQL multiple result sets" do
     before do
       MYSQL_DB.create_table!(:a){Integer :a}
       MYSQL_DB.create_table!(:b){Integer :b}
