@@ -6,30 +6,55 @@ describe "Database schema parser" do
     @iom = INTEGRATION_DB.identifier_output_method
     @iim = INTEGRATION_DB.identifier_input_method
     @defsch = INTEGRATION_DB.default_schema
-    clear_sqls
+    @qi = INTEGRATION_DB.quote_identifiers?
   end
   after do
     INTEGRATION_DB.identifier_output_method = @iom
     INTEGRATION_DB.identifier_input_method = @iim
     INTEGRATION_DB.default_schema = @defsch
+    INTEGRATION_DB.quote_identifiers = @qi
     INTEGRATION_DB.drop_table(:items) if INTEGRATION_DB.table_exists?(:items)
   end
 
-  specify "should handle a database with a identifier_output_method" do
+  specify "should handle a database with a identifier methods" do
     INTEGRATION_DB.identifier_output_method = :reverse
     INTEGRATION_DB.identifier_input_method = :reverse
+    INTEGRATION_DB.quote_identifiers = true
     INTEGRATION_DB.default_schema = nil if INTEGRATION_DB.default_schema
     INTEGRATION_DB.create_table!(:items){Integer :number}
-    INTEGRATION_DB.schema(:items, :reload=>true).should be_a_kind_of(Array)
-    INTEGRATION_DB.schema(:items, :reload=>true).first.first.should == :number
+    begin
+      INTEGRATION_DB.schema(:items, :reload=>true).should be_a_kind_of(Array)
+      INTEGRATION_DB.schema(:items, :reload=>true).first.first.should == :number
+    ensure 
+      INTEGRATION_DB.drop_table(:items)
+    end
+  end
+
+  specify "should handle a dataset with identifier methods different than the database's" do
+    INTEGRATION_DB.identifier_output_method = :reverse
+    INTEGRATION_DB.identifier_input_method = :reverse
+    INTEGRATION_DB.quote_identifiers = true
+    INTEGRATION_DB.default_schema = nil if INTEGRATION_DB.default_schema
+    INTEGRATION_DB.create_table!(:items){Integer :number}
+    INTEGRATION_DB.identifier_output_method = @iom
+    INTEGRATION_DB.identifier_input_method = @iim
+    ds = INTEGRATION_DB[:items]
+    ds.identifier_output_method = :reverse
+    ds.identifier_input_method = :reverse
+    begin
+      INTEGRATION_DB.schema(ds, :reload=>true).should be_a_kind_of(Array)
+      INTEGRATION_DB.schema(ds, :reload=>true).first.first.should == :number
+    ensure 
+      INTEGRATION_DB.identifier_output_method = :reverse
+      INTEGRATION_DB.identifier_input_method = :reverse
+      INTEGRATION_DB.drop_table(:items)
+    end
   end
 
   specify "should not issue an sql query if the schema has been loaded unless :reload is true" do
     INTEGRATION_DB.create_table!(:items){Integer :number}
     INTEGRATION_DB.schema(:items, :reload=>true)
-    clear_sqls
     INTEGRATION_DB.schema(:items)
-    clear_sqls
     INTEGRATION_DB.schema(:items, :reload=>true)
   end
 
@@ -56,11 +81,10 @@ describe "Database schema parser" do
     col_info = col.last
     col_info.should be_a_kind_of(Hash)
     col_info[:type].should == :integer
-    clear_sqls
     INTEGRATION_DB.schema(:items)
   end
 
-  cspecify "should parse primary keys from the schema properly", [proc{|db| db.adapter_scheme != :jdbc}, :mssql] do
+  specify "should parse primary keys from the schema properly" do
     INTEGRATION_DB.create_table!(:items){Integer :number}
     INTEGRATION_DB.schema(:items).collect{|k,v| k if v[:primary_key]}.compact.should == []
     INTEGRATION_DB.create_table!(:items){primary_key :number}
@@ -85,7 +109,7 @@ describe "Database schema parser" do
     INTEGRATION_DB.schema(:items).first.last[:ruby_default].should == 'blah'
   end
 
-  specify "should parse types from the schema properly" do
+  cspecify "should parse types from the schema properly", [:jdbc, :db2], :oracle do
     INTEGRATION_DB.create_table!(:items){Integer :number}
     INTEGRATION_DB.schema(:items).first.last[:type].should == :integer
     INTEGRATION_DB.create_table!(:items){Fixnum :number}
@@ -156,7 +180,6 @@ describe "Database schema modifiers" do
   before do
     @db = INTEGRATION_DB
     @ds = @db[:items]
-    clear_sqls
   end
   after do
     @db.drop_table(:items) if @db.table_exists?(:items)
@@ -174,6 +197,24 @@ describe "Database schema modifiers" do
     @db.create_table!(:items, :temp=>true){Integer :number}
   end
   
+  specify "should have create_table? only create the table if it doesn't already exist" do
+    @db.create_table!(:items){String :a}
+    @db.create_table?(:items){String :b}
+    @db[:items].columns.should == [:a]
+    @db.drop_table(:items) rescue nil
+    @db.create_table?(:items){String :b}
+    @db[:items].columns.should == [:b]
+  end
+
+  specify "should have create_table? work correctly with indexes" do
+    @db.create_table!(:items){String :a, :index=>true}
+    @db.create_table?(:items){String :b, :index=>true}
+    @db[:items].columns.should == [:a]
+    @db.drop_table(:items) rescue nil
+    @db.create_table?(:items){String :b, :index=>true}
+    @db[:items].columns.should == [:b]
+  end
+
   specify "should rename tables correctly" do
     @db.drop_table(:items) rescue nil
     @db.create_table!(:items2){Integer :number}
@@ -193,6 +234,13 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:number]
   end
 
+  specify "should handle combination of default, unique, and not null" do
+    @db.create_table!(:items){Integer :number, :default=>0, :null=>false, :unique=>true}
+    @db.table_exists?(:items).should == true
+    @db.schema(:items, :reload=>true).map{|x| x.last}.first.values_at(:ruby_default, :allow_null).should == [0, false]
+    @ds.insert([10])
+  end
+
   specify "should handle foreign keys correctly when creating tables" do
     @db.create_table!(:items) do 
       primary_key :id
@@ -208,24 +256,34 @@ describe "Database schema modifiers" do
   specify "should add columns to tables correctly" do
     @db.create_table!(:items){Integer :number}
     @ds.insert(:number=>10)
-    @db.alter_table(:items){add_column :name, :text}
+    @db.alter_table(:items){add_column :name, String}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:number, :name]
     @ds.columns!.should == [:number, :name]
     @ds.all.should == [{:number=>10, :name=>nil}]
   end
 
-  cspecify "should add primary key columns to tables correctly", :h2 do
+  cspecify "should add primary key columns to tables correctly", :h2, :derby do
     @db.create_table!(:items){Integer :number}
     @ds.insert(:number=>10)
     @db.alter_table(:items){add_primary_key :id}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:number, :id]
     @ds.columns!.should == [:number, :id]
     @ds.map(:number).should == [10]
+    proc{@ds.insert(:id=>@ds.map(:id).first)}.should raise_error
+  end
+
+  specify "should drop primary key constraints from tables correctly" do
+    @db.create_table!(:items){Integer :number; primary_key [:number], :name=>:items_pk}
+    @ds.insert(:number=>10)
+    @db.alter_table(:items){drop_constraint :items_pk, :type=>:primary_key}
+    @ds.map(:number).should == [10]
+    proc{@ds.insert(10)}.should_not raise_error
   end
 
   specify "should add foreign key columns to tables correctly" do
     @db.create_table!(:items){primary_key :id}
-    i = @ds.insert
+    @ds.insert
+    i = @ds.get(:id)
     @db.alter_table(:items){add_foreign_key :item_id, :items}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :item_id]
     @ds.columns!.should == [:id, :item_id]
@@ -262,7 +320,7 @@ describe "Database schema modifiers" do
     proc{@ds.insert(:n=>nil)}.should raise_error(Sequel::DatabaseError)
   end
 
-  specify "should set column NULL/NOT NULL correctly" do
+  cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2], [:db2] do
     @db.create_table!(:items, :engine=>:InnoDB){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_allow_null :id, false}
@@ -270,7 +328,7 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:id]
     proc{@ds.insert(:id=>nil)}.should raise_error(Sequel::DatabaseError)
     @db.alter_table(:items){set_column_allow_null :id, true}
-    @ds.insert
+    @ds.insert(:id=>nil)
     @ds.all.should == [{:id=>10}, {:id=>nil}]
   end
 
@@ -284,27 +342,60 @@ describe "Database schema modifiers" do
     @ds.all.should == [{:id=>10}, {:id=>20}]
   end
 
-  specify "should set column types correctly" do
+  cspecify "should set column types correctly", [:jdbc, :db2], [:db2], :oracle do
     @db.create_table!(:items){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_type :id, String}
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id]
     @ds.columns!.should == [:id]
-    @ds.insert(:id=>20)
+    @ds.insert(:id=>'20')
     @ds.all.should == [{:id=>"10"}, {:id=>"20"}]
   end
 
-  cspecify "should add unique constraints and foreign key table constraints correctly", :sqlite do
-    @db.create_table!(:items){Integer :id; Integer :item_id}
+  specify "should add unnamed unique constraints and foreign key table constraints correctly" do
+    @db.create_table!(:items, :engine=>:InnoDB){Integer :id; Integer :item_id}
     @db.alter_table(:items) do
       add_unique_constraint [:item_id, :id]
       add_foreign_key [:id, :item_id], :items, :key=>[:item_id, :id]
     end
     @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :item_id]
     @ds.columns!.should == [:id, :item_id]
+    proc{@ds.insert(1, 1)}.should_not raise_error
+    proc{@ds.insert(1, 1)}.should raise_error
+    proc{@ds.insert(1, 2)}.should raise_error
   end
 
-  cspecify "should remove columns from tables correctly", :h2, :mssql do
+  specify "should add named unique constraints and foreign key table constraints correctly" do
+    @db.create_table!(:items, :engine=>:InnoDB){Integer :id, :null=>false; Integer :item_id, :null=>false}
+    @db.alter_table(:items) do
+      add_unique_constraint [:item_id, :id], :name=>:unique_iii
+      add_foreign_key [:id, :item_id], :items, :key=>[:item_id, :id], :name=>:fk_iii
+    end
+    @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :item_id]
+    @ds.columns!.should == [:id, :item_id]
+    proc{@ds.insert(1, 1)}.should_not raise_error
+    proc{@ds.insert(1, 1)}.should raise_error
+    proc{@ds.insert(1, 2)}.should raise_error
+  end
+
+  specify "should drop unique constraints and foreign key table constraints correctly" do
+    @db.create_table!(:items) do
+      Integer :id
+      Integer :item_id
+      unique [:item_id, :id], :name=>:items_uk
+      foreign_key [:id, :item_id], :items, :key=>[:item_id, :id], :name=>:items_fk
+    end
+    @db.alter_table(:items) do
+      drop_constraint(:items_fk, :type=>:foreign_key)
+      drop_constraint(:items_uk, :type=>:unique)
+    end
+    @db.schema(:items, :reload=>true).map{|x| x.first}.should == [:id, :item_id]
+    @ds.columns!.should == [:id, :item_id]
+    proc{@ds.insert(1, 2)}.should_not raise_error
+    proc{@ds.insert(1, 2)}.should_not raise_error
+  end
+
+  cspecify "should remove columns from tables correctly", :h2, :mssql, [:jdbc, :db2], :hsqldb do
     @db.create_table!(:items) do
       primary_key :id
       String :name
@@ -325,7 +416,7 @@ describe "Database schema modifiers" do
     @ds.columns!.should == [:id]
   end
 
-  specify "should remove multiple columns in a single alter_table block" do
+  cspecify "should remove multiple columns in a single alter_table block", [:jdbc, :db2] do
     @db.create_table!(:items) do
       primary_key :id
       String :name
@@ -355,25 +446,72 @@ describe "Database#tables" do
         "xxxxx#{@@xxxxx += 1}"
       end
     end
-    @iom = INTEGRATION_DB.identifier_output_method
-    @iim = INTEGRATION_DB.identifier_input_method
-    clear_sqls
+    @db = INTEGRATION_DB
+    @db.create_table(:sequel_test_table){Integer :a}
+    @db.create_view :sequel_test_view, @db[:sequel_test_table]
+    @iom = @db.identifier_output_method
+    @iim = @db.identifier_input_method
   end
   after do
-    INTEGRATION_DB.identifier_output_method = @iom
-    INTEGRATION_DB.identifier_input_method = @iim
+    @db.identifier_output_method = @iom
+    @db.identifier_input_method = @iim
+    @db.drop_view :sequel_test_view
+    @db.drop_table :sequel_test_table
   end
 
   specify "should return an array of symbols" do
-    ts = INTEGRATION_DB.tables
+    ts = @db.tables
     ts.should be_a_kind_of(Array)
     ts.each{|t| t.should be_a_kind_of(Symbol)}
+    ts.should include(:sequel_test_table)
+    ts.should_not include(:sequel_test_view)
   end
 
   specify "should respect the database's identifier_output_method" do
-    INTEGRATION_DB.identifier_output_method = :xxxxx
-    INTEGRATION_DB.identifier_input_method = :xxxxx
-    INTEGRATION_DB.tables.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
+    @db.identifier_output_method = :xxxxx
+    @db.identifier_input_method = :xxxxx
+    @db.tables.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
+  end
+end
+end
+
+begin
+  INTEGRATION_DB.views
+rescue Sequel::NotImplemented
+rescue
+describe "Database#views" do
+  before do
+    class ::String
+      @@xxxxx = 0
+      def xxxxx
+        "xxxxx#{@@xxxxx += 1}"
+      end
+    end
+    @db = INTEGRATION_DB
+    @db.create_table(:sequel_test_table){Integer :a}
+    @db.create_view :sequel_test_view, @db[:sequel_test_table]
+    @iom = @db.identifier_output_method
+    @iim = @db.identifier_input_method
+  end
+  after do
+    @db.identifier_output_method = @iom
+    @db.identifier_input_method = @iim
+    @db.drop_view :sequel_test_view
+    @db.drop_table :sequel_test_table
+  end
+
+  specify "should return an array of symbols" do
+    ts = @db.views
+    ts.should be_a_kind_of(Array)
+    ts.each{|t| t.should be_a_kind_of(Symbol)}
+    ts.should_not include(:sequel_test_table)
+    ts.should include(:sequel_test_view)
+  end
+
+  specify "should respect the database's identifier_output_method" do
+    @db.identifier_output_method = :xxxxx
+    @db.identifier_input_method = :xxxxx
+    @db.views.each{|t| t.to_s.should =~ /\Ax{5}\d+\z/}
   end
 end
 end
